#!/usr/bin/perl

# dircast.cgi
#   ディレクトリ内のファイルをポッドキャスティング
#   2007/3/9 Nihondo.
#
# 実行方法
#   プログラムID "yukarin" の番組のRSSを出力する（http経由）
#     http://192.168.1.1/cgi-bin/dircast.cgi?yukarin
#
#   コマンドラインから実行した場合、設定したディレクトリに yukarin.xml を出力する
#     perl dircast.cgi yukarin
#
#   プログラムIDに"all" を指定した場合は、全プログラムについて出力する


# 設定ここから

	# ポッドキャストのタイトルを設定
	$podcast_title = "radioSHARK Podcasting";

	# ポッドキャストの説明を設定
	$podcast_desc = "radioSHARK で録音したラジオ番組をポッドキャストします。";

	# 配信したいファイルがあるディレクトリをフルパスで設定
	#   http経由でアクセスできるディレクトリを設定すること
	#   Windows環境では、ディレクトリ名および、配信するファイル名に日本語を含まないようにすること
	#     $dirpath = "c:/Program Files/Apache Software Foundation/Apache2.2/htdocs/radioSHARK";	#Windowsの例
	$dirpath = "/Users/Nihondo/Sites/radioSHARK";
	
	# 配信したいファイルがあるディレクトリのURL（$dirpathをURLで表記）
	$dirurl = "http://silk.local/~Nihondo/radioSHARK";

	# RSSのファイルを出力するディレクトリを設定（コマンドラインから実行したときのみ有効）
	#   [プログラムID].xml というファイル名でRSSが出力される
	#   http経由でアクセスできるディレクトリを設定すること
	$outputdir = "/Users/Nihondo/Sites";

	# プログラムの設定
	#   プログラムIDは一意になるIDでcgi実行時の引数として使う
	#   title：番組名
	#   desc：番組の説明
	#   keyword：このキーワードを含むファイルをディレクトリ内から抽出
	#   replace：指定した場合は、keyword を replace の文字列で置き換える
	#
	#（例）
	#   "yukarin" => {		# プログラムID
	#   	"title"		=> "田村ゆかりのいたずら黒うさぎ",		# 番組名（ポッドキャストのタイトルになる）
	#   	"desc"		=> "文化放送「田村ゆかりのいたずら黒うさぎ」 土曜・24:00～24:30",
	#   	"keyword"	=> "kurousagi",		# $dirpathディレクトリ配下のファイル「kurousagi - Mar 9, 2007.m4a」などを抽出
	#   	"replace"	=> "田村ゆかりのいたずら黒うさぎ"	# 「田村ゆかりのいたずら黒うさぎ - Mar 9, 2007」と置き換え
	#	},

	%program = (
		"yukarin" => {
			"title"		=> "田村ゆかりのいたずら黒うさぎ",
			"desc"		=> "文化放送「田村ゆかりのいたずら黒うさぎ」 土曜・24:00～24:30",
			"keyword"	=> "kurousagi",
			"replace"	=> "田村ゆかりのいたずら黒うさぎ"
		},
		"nana" => {
			"title"		=> "水樹奈々スマイル・ギャング",
			"desc"		=> "文化放送「水樹奈々スマイル・ギャング」 木曜・21:30～22:00",
			"keyword"	=> "smilegang",
			"replace"	=> "水樹奈々スマイル・ギャング"
		},
		"marunavi" => {
			"title"		=> "PONY CANYON STYLE まるなび！？",
			"desc"		=> "文化放送「PONY CANYON STYLE まるなび！？」 火曜・25:00～25:30",
			"keyword"	=> "marunavi",
			"replace"	=> "PONY CANYON STYLE まるなび！？"
		},
		"anispa" => {
			"title"		=> "A&G超RADIO SHOW～アニスパ！～",
			"desc"		=> "文化放送「A&G超RADIO SHOW～アニスパ！～」 土曜・21:00～23:00",
			"keyword"	=> "anispa",
			"replace"	=> "A&G超RADIO SHOW～アニスパ！～"
		},
	);

# 以下はお好みで設定

	# ファイルサイズ監視実行のしきい値
	#   更新中ファイルがリストされるのを避けるため、ファイルの最終更新時間が以下の時間（分）以内の間は、
	#   ファイルサイズをチェックし、サイズの変動を検知した場合はリストしない
	$delay_time = 10;

	# 配信を許可するファイルの拡張子とファイルタイプを設定
	%file_types = (
		"mp3" => "audio/x-mp3",
		"m4a" => "audio/x-m4a",
		"m4v" => "video/x-m4v",
		"mp4" => "video/x-mp4",
		"wma" => "audio/x-ms-wma",
		"wmv" => "video/x-ms-wmv",
	);

# 設定ここまで



# 引数からプログラムIDを取得
if(@ARGV > 0){
	$parm = @ARGV[@ARGV-1];
}

if( $parm ne "" && $parm ne "all"){
	if( !exists($program{$parm}) ){
		# プログラムIDが存在しない場合は処理しない
		exit;
	}
	$podcast_title = $program{$parm}{"title"}." : ".$podcast_title;
}else{
	$parm = "all";
}

# RSS作成の準備
my $rss = new( $rss,
	version => "1.0",
	encoding => "UTF-8"
);

channel( $rss,
	title => $podcast_title,
	description  => $podcast_desc,
);

# $dirpath ディレクトリ内のファイルを一つずつチェックする
opendir(DIR, "$dirpath");
while ($file = readdir(DIR)) {
	# ディレクトリをはじく
	next if -d "$dirpath/$file";

	# 拡張子のリストに存在しない拡張子を含むファイルをはじく
	@filename = split(/\./, $file);
	$ext = @filename[@filename-1];
	next if !exists($file_types{lc($ext)});

	# keyword で指定した文字列が含まれているファイル以外をはじく
	$programID = $parm;
	if( $programID eq "all" ){
		# プログラムID未指定の場合は、すべての keyword を試す
		foreach $keyword (keys %program){
			next if $file !~ /$program{$keyword}{"keyword"}/i;
			$programID = $keyword;
		}
		next if $programID eq "all";
	}else{
		next if $file !~ /$program{$programID}{"keyword"}/i;
	}

	# ファイルサイズの取得
	$size = -s "$dirpath/$file";
	# 最終更新時間の取得
	$fixtime = (stat("$dirpath/$file"))[9];
	# $delay_time 以内に更新されたファイルは、3秒後のファイルサイズに変化があればはじく
	if( $delay_time > 0 && time-$fixtime < 60*$delay_time ){
		sleep 3;
		next if $size != -s "$dirpath/$file";
	}

	# ファイル名から拡張子を取り除いた部分を取得し、タイトル文字列とする
	pop(@filename);
	$title = join(/./, @filename);

	# タイトル文字列をURLエンコード
	$url = $title;
	$url =~ s/(\W)/'%'.unpack("H2", $1)/ego;
	$url =~ tr/ /+/;

	# replace が指定されている場合は、タイトル文字列の置き換えを行う
	if( $program{$programID}{"replace"} ne "" ){
		$title =~ s/$program{$programID}{"keyword"}/$program{$programID}{"replace"}/;
	}

	# <pubDate>の日付文字列を作成
	($sec,$min,$hour,$day,$month,$year,$wday) = localtime($fixtime);
	$datestr = sprintf("%s, %2d %s %4d %02d:%02d:%02d +0900",
		("Sun","Mon","Tue","Wed","Thu","Fri","Sat")[$wday], $day,
		("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec")[$month], $year+1900,
		$hour, $min, $sec
	);

	# RSSアイテムを追加
	add_item($rss,
	    title => $title,
	    description => $program{$programID}{"desc"},
	    url => "$dirurl/$url.$ext",
	    length => "$size",
	    type => $file_types{lc($ext)},
	    pubDate => "$datestr",
	);
}
closedir(DIR);

# http、またはファイルへRSSを出力
if($ENV{'QUERY_STRING'} ne ""){
	print "Content-type: text/html\n\n";
	print as_string($rss);
}else{
	open(OUT, ">$outputdir/$parm.xml");
	print OUT as_string($rss);
	close(OUT);
}

exit;



# RSS Perser based on Yuki::RSS
# http://www.hyuki.com/techinfo/yukirss.html

# The constructor.
sub new {
	my ($class, %hash) = @_;
	my $self = {
		version => $hash{version},
		encoding => $hash{encoding},
		channel => { },
		items => [],
	};
	return bless $self, $class;
}

# Setting channel.
sub channel {
	my ($self, %hash) = @_;
	foreach (keys %hash) {
		$self->{channel}->{$_} = $hash{$_};
	}
	return $self->{channel};
}

# Adding item.
sub add_item {
	my ($self, %hash) = @_;
	push(@{$self->{items}}, \%hash);
	return $self->{items};
}

sub as_string {
	my ($self) = @_;
	my $about = $self->{channel}->{about};
	$about = $self->{channel}->{link} unless ($about);
	my $doc = <<"EOD";
<?xml version="1.0" encoding="$self->{encoding}" ?>

<rss
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:content="http://purl.org/rss/1.0/modules/content/"
	xmlns:dcterms="http://purl.org/rss/1.0/modules/dcterms/"
	version="2.0"
>

<channel>

	<title>$self->{channel}->{title}</title>
	<description>$self->{channel}->{description}</description>
@{[
map {
qq{
	<item>
		<title>$_->{title}</title>
		<description>$_->{description}</description>
		<enclosure url="$_->{url}" length="$_->{length}" type="$_->{type}" />
		<pubDate>$_->{pubDate}</pubDate> 
	</item>
}
} @{$self->{items}}
]}
</channel>

</rss>
EOD
}
